#!/bin/bash
#

# buildGCC.sh <product_dir> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch

}

die()
{
  echo "$1"; exit 2
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr}"
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -z ${product_dir}
    fi
}

unpack_and_patch()
{
    local tardest=${blddir}/src
    mkdir -p ${tardest}

    # unpack
    gcctar=${tardir}/gcc-${pkgdotver}.tar.bz2
    if [[ -d "${tardest}/gcc-${pkgdotver}" ]]; then
      echo "GCC source already exists."
    else
      tar -xf ${gcctar} -C ${blddir}/src || die "Unable to untar gcc source into ${blddir}/src"
    fi
    tardest="${tardest}/gcc-${pkgdotver}"
    for tarfile in "${tardir}/"*.tar.*; do
      [[ "${tarfile}" == "${gcctar}" ]] && continue
      tarfile=$(basename "${tarfile}")
      subpkg=${tarfile%%-*}
      subpkgdir=${tarfile%.tar*}
      subpkgdir=${subpkgdir%%[a-z]}
      if [[ -d "${tardest}/${subpkg}" ]]; then
        echo "${subpkg} source already exists in ${tardest}"
      else
        rm -rf "${tardest}/${subpkgdir}"
        { tar -xf "${tardir}/${tarfile}" -C "${tardest}" && mv "${tardest}/${subpkgdir}" "${tardest}/${subpkg}"; } \
         || die "Unable to untar ${subpkg} source into ${tardest}"
      fi
      if [[ -f "${patchdir}/${subpkgdir}.patch" ]]; then
        ( cd "${tardest}/${subpkgdir}" && patch -b -p0 --verbose < "${patchdir}/${subpkgdir}.patch" || die "Unable to apply patch ${subpkgdir}.patch to ${subpkg}" )
      fi
    done
    flvr1=`ups flavor -1`
    if [ ${flvr1} = "Darwin" ]
    then
	set -x
	cd ${blddir}/src
	perl -pi.bak -e 's/uname -p/uname -m/go;' */config.guess */configfsf.guess
	set +x
    fi
}

gcc_built() {
  [[ -f ${pkgdir}/lib/libstdc++.${shext} ]] && [[ -f ${pkgdir}/lib/libgfortran.${shext} ]] && \
   [[ -x ${pkgdir}/bin/gcc ]] && [[ -x ${pkgdir}/bin/g++ ]] && [[ -x ${pkgdir}/bin/gfortran ]] && \
   [[ -d ${pkgdir}/include/c++ ]]
}
# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gcc
pkgver=v4_9_1
ssibuildshims_version=v0_07_04
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

get_platform_name

if [ ${OS1} = "Darwin" ]
then
  echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
  export DYLD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=dylib
else
  echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
  export LD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=so
fi

if [ "${maketar}" = "tar" ] && gcc_built
then
    ups exist ${package} ${pkgver} || declare_product
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi


unpack_and_patch

# set the build environment
# set CC and CXX
export CC=gcc
export CXX=g++
num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)
# remove /usr/local/bin and . from path
PATH=`dropit /usr/local/bin .`
# remove /usr/local/X from LD_LIBRARY PATH and DYLD_LIBRARY_PATH
LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+$(dropit -a -p "$LD_LIBRARY_PATH" /usr/local/)}
DYLD_LIBRARY_PATH=${DYLD_LIBRARY_PATH:+$(dropit -a -p "$DYLD_LIBRARY_PATH" /usr/local/)}
# make sure libraries are in the path
echo "COMPILER_PATH   $COMPILER_PATH"
export PATH=${pkgdir}/bin:${PATH}
export COMPILER_PATH=${pkgdir}/bin
# need proper environment
echo ""
echo "CC $CC CXX $CXX "
echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
echo "COMPILER_PATH   $COMPILER_PATH"
# build gcc
if gcc_built
then
    echo "gcc has already been built"
else
    set -x
    cd ${blddir}/src/gcc-*
#    export CXXFLAGS="-fPIC"
#    export CFLAGS="-fPIC"
#    enable_watchdog=yes \
     ./configure --prefix=${pkgdir} \
                 --disable-libgcj \
                 --enable-threads=posix \
                 --enable-__cxa_atexit \
                 --enable-languages=c,c++,objc,obj-c++,fortran \
                 --with-system-zlib \
                 --enable-libstdcxx-time=yes \
                 --enable-stage1-checking \
                 --enable-checking=release \
                 --enable-lto \
                 --enable-plugin \
     || \
     die "ERROR: Unable to configure gcc."
    make bootstrap MAKE="make -j ${num_cores}" -j "${num_cores}" || die "ERROR: Unable to make gcc."
    make install || die "ERROR: Unable to install gcc."
    set +x
fi
if ! gcc_built
then
   echo "ERROR: the gcc build has failed"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Remove .la files.
find "${pkgdir}" -name '*.la' | xargs rm -vf

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}
echo "${package} is installed at ${GCC_FQ_DIR}"


# this must be last because it will exit
if [ "${maketar}" = "tar" ] && gcc_built
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
fi

exit 0
